// Load Panel for Standardized File Loaders
// Jeffrey J Weimer
// SVN History: $Revision: 801 $ on $Date: 2008-07-07 11:01:25 -0500 (Mon, 07 Jul 2008) $

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=2.0

#pragma ModuleName=udFLStandardPanel

#include "udFLStandardFunctions", version >=2.0
#include "udFLStandardPanel.lcl"

// set to 0 normally, 1 for testing

Static Constant TestPanel=1

// panel information

Static Constant PWidth=235
Static Constant PHeight=330
Static Constant SPWidth=230
Static Constant SPHeight=250
Static Constant PTrim=10
Static StrConstant kPanelVerStr="v 2.0"

// menu

Menu "Misc", hideable
	SubMenu "Panels"
		"Standardized File Loaders Panel", /Q, udFLStandardPanel#ShowPanel(0)
	End
End

// Show Panel

Static Function ShowPanel(restart)
	variable restart
	
	// kill or bring forward panel
	
	if (strlen(WinList("udFL_Panel","","WIN:64"))!=0)
		if (restart)
			KillWindow udFL_Panel
		else
			DoWindow/F udFL_Panel
			return 0
		endif
	endif
	
	// (re)initialize globals 
	
	string cdf = GetDataFolder(1)
	
	NewDataFolder/O/S root:Packages
	NewDataFolder/O/S StandardizedFileLoader
	
	string/G loadtofolder="", loadfileprefix="", userData="", loadfunctionname=""
	string/G udFLdatapath="", listofwavesloaded=""
	
	PathInfo udFL_DataFiles
	if (V_flag)
		udFLdatapath=StringFromList(ItemsInList(S_path,":")-1,S_path,":")
	endif
	make/O/T/N=0 filesindatapath
	make/O/N=0 nfilesindatapath
	
	SetDataFolder $cdf
	
	// initialize new panel and screen sizes

#if(exists("ScreenSizer#Initialize")==6)
	ScreenSizer#Initialize(0)
	ScreenSizer#MakePanel(-5,5,PWidth,PHeight,"Data",other="/K=1/N=udFL_Panel")
#else
	NewPanel/K=1/N=udFL_Panel/W=(2,2,PWidth,PHeight) as "Data"
#endif
	if (!TestPanel)
		ModifyPanel fixedSize=1, noEdit=1
	endif
	
	// Tabs
	
	TabControl tb, pos={1,2},size={PWidth,20}, proc=udFLStandardPanel#ShowPanelTab
	TabControl tb, tabLabel(0)=udFLpan00, value=0
	
	TitleBox tb_udFLTitle title=kPanelVerStr, fSize=8, pos={200,5}, frame=0
	
	// Controls
	
	PanelBuildTab(0)
	PanelBuildTab(99)
	
	// finalize new panel
	
	PanelFillTab(0)

	return 1
End

// build a tab on the data panel

Static Function PanelBuildTab(tab)
	variable tab
	
	STRUCT udFLStandardStructure udFL
	udFLZeroStructure(udFL)

	// Control Globals and Mode Settings
		
	SVAR ltf = root:Packages:StandardizedFileLoader:loadtofolder
	SVAR lfp = root:Packages:StandardizedFileLoader:loadfileprefix
	SVAR uD = root:Packages:StandardizedFileLoader:userData
	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	SVAR thePath = root:Packages:StandardizedFileLoader:udFLdatapath

	// build a specific tab
	
	switch(tab)
	
		case 0:
	
			lfN = StringFromList(0,udFLLoaderList(1))
		
			// Load Tab
			// =====
		
//			// Folder List
//			
//			PopupMenu flist_tb0,pos={PTrim,39},size={122,20},title=udFLpan00a,fSize=12
//			PopupMenu flist_tb0, mode=1,popvalue="root", value=#"udFLStandardPanel#FolderList()"
//			PopupMenu flist_tb0, proc=udFLStandardPanel#CreateNewFolderName
//		
//			// CreateNew Input
//		
//			SetVariable newdir_tb0,pos={PTrim,65},size={PWidth-2*PTrim-10,17},title="root:",fSize=12
//			SetVariable newdir_tb0,value=ltf, proc=udFLStandardPanel#ValidateLoadFolderName
//			SetVariable newdir_tb0, help={"You may NOT use : or space characters here!"}
//
//			// Data Folder Exists Warning
//
//			TitleBox fldrx_tb0,pos={PWidth-2*PTrim+5,65},size={15,15},title="!",fSize=12
//			TitleBox fldrx_tb0, help={"This data folder already exists!"}
//		
//			// FileName
//		
//			PopupMenu fname_tb0,pos={PTrim,92},size={87,20},title=udFLpan00c,fSize=12
//			PopupMenu fname_tb0,mode=1,value=#"udFLStandardPanel#RenameInputList()"
//			PopupMenu fname_tb0, proc=udFLStandardPanel#CreateNewTraceName
//
//			// CreateNew Filename
//		
//			SetVariable newfname_tb0,pos={PTrim,118},size={PWidth-2*PTrim,17},title="input:",fSize=12
//			SetVariable newfname_tb0,value=lfp, proc=udFLStandardPanel#ValidateNewTraceName
//			SetVariable newfname_tb0, help={"You may NOT use certain characters here!"}
//
//			// Loader
//		
//			PopupMenu loader_tb0,pos={PTrim,148},size={87,20},title=udFLpan00d,fSize=12
//			PopupMenu loader_tb0,mode=1,value=#"udFLStandardPanel#udFLLoaderList(0)"
//			PopupMenu loader_tb0, proc=udFLStandardPanel#ChangeLoaderModes
//	
//			//  Loading Modes
//			
//			PopupMenu lmode_tb0,pos={PTrim,175},size={87,20},title=udFLpan00e,fSize=12
//			PopupMenu lmode_tb0, mode=1, value=#"udFLStandardPanel#FileLoaderModeList()"
//
//			// Report Modes
//			
//			PopupMenu rpmode_tb0,pos={PWidth-87,175},size={87,20},title="",fSize=12
//			PopupMenu rpmode_tb0, mode=2, value=udFLpan00h
//	
//			//  Processing Modes
//			
//			PopupMenu pmode_tb0,pos={PTrim,200},size={87,20},title=udFLpan00f,fSize=12
//			PopupMenu pmode_tb0, mode=1, value=#"udFLStandardPanel#FileProcessModeList()"
//
//			// User Data
//			
//			SetVariable userData_tb0,pos={PTrim,227},size={PWidth-2*PTrim,17},title=udFLpan00g,fSize=12
//			SetVariable userData_tb0,value=uD
//			SetVariable userData_tb0, help={"Each file loader has its own User Data input"}

			// Folder List
			
			PopupMenu flist_tb0,pos={PTrim,30},size={122,20},title=udFLpan00a,fSize=12
			PopupMenu flist_tb0, mode=1,popvalue="root", value=#"udFLStandardPanel#FolderList()"
			PopupMenu flist_tb0, proc=udFLStandardPanel#CreateNewFolderName
		
			// CreateNew Input
		
			SetVariable newdir_tb0,pos={PTrim,55},size={PWidth-2*PTrim-10,17},title="root:",fSize=12
			SetVariable newdir_tb0,value=ltf, proc=udFLStandardPanel#ValidateLoadFolderName
			SetVariable newdir_tb0, help={"You may NOT use : or space characters here!"}

			// Data Folder Exists Warning

			TitleBox fldrx_tb0,pos={PWidth-2*PTrim+5,53},size={15,15},title="!",fSize=12
			TitleBox fldrx_tb0, help={"This data folder already exists!"}	

			// Loader
		
			PopupMenu loader_tb0,pos={PTrim,85},size={87,20},title=udFLpan00d,fSize=12
			PopupMenu loader_tb0,mode=1,value=#"udFLStandardPanel#udFLLoaderList(0)"
			PopupMenu loader_tb0, proc=udFLStandardPanel#ChangeLoaderModes

			//  Loading Modes
			
			PopupMenu lmode_tb0,pos={PTrim,110},size={87,20},title=udFLpan00e,fSize=12
			PopupMenu lmode_tb0, mode=1, value=#"udFLStandardPanel#FileLoaderModeList()"

			// Report Modes
			
			PopupMenu rpmode_tb0,pos={PWidth-87,110},size={87,20},title="",fSize=12
			PopupMenu rpmode_tb0, mode=2, value=udFLpan00h

			//  Processing Modes
			
			PopupMenu pmode_tb0,pos={PTrim,135},size={87,20},title=udFLpan00f,fSize=12
			PopupMenu pmode_tb0, mode=1, value=#"udFLStandardPanel#FileProcessModeList()"
	
			// User Data
			
			SetVariable userData_tb0,pos={PTrim,160},size={PWidth-2*PTrim,17},title=udFLpan00g,fSize=12
			SetVariable userData_tb0,value=uD
			SetVariable userData_tb0, help={"Each file loader has its own User Data input"}
		
			// FileName
		
			PopupMenu fname_tb0,pos={PTrim,190},size={87,20},title=udFLpan00c,fSize=12
			PopupMenu fname_tb0,mode=1,value=#"udFLStandardPanel#RenameInputList()"
			PopupMenu fname_tb0, proc=udFLStandardPanel#CreateNewTraceName

			// CreateNew Filename
		
			SetVariable newfname_tb0,pos={PTrim,215},size={PWidth-2*PTrim,17},title="input:",fSize=12
			SetVariable newfname_tb0,value=lfp, proc=udFLStandardPanel#ValidateNewTraceName
			SetVariable newfname_tb0, help={"You may NOT use certain characters here!"}

			// SetPaths
			
			Button setpaths_tb0,pos={PTrim,255},size={85,25}, proc=udFLStandardPanel#SetDirPaths
			Button setpaths_tb0,fSize=12, fColor=(49151,49151,65535), userdata="0"
			SetVariable lpath_tb0,pos={PTrim,285},size={110,16},title=" ",fSize=10
			SetVariable lpath_tb0,value=thePath,noedit= 1
			
			PathInfo udFL_DataFiles
			if (V_flag==0)
				Button setpaths_tb0, title=udFLpan00i1
			else
				Button setpaths_tb0, title=udFLpan00i2, userdata="1"
			endif
			break
		
		default:

			// Close Panel When Complete
		
			CheckBox tb_closeme,pos={PTrim+15,310},size={139,14},title=udFLpan00j, fSize=10
			CheckBox tb_closeme,value= 0
			
			// Do It!
			
			Button tb_doit,pos={PTrim+120,260},size={70,20},proc=udFLStandardPanel#DoIt
			Button tb_doit, font="Arial",fSize=14, fColor=(49151,49151,65535)
			break
			
	endswitch
	
	return 0
	
End

// fill a tab when it is selected

Static Function PanelFillTab(tab)
	variable tab
	
	string tStr
	
	tStr = "!*_tb"+num2str(tab)	
	ModifyControlList ControlNameList("",";",tStr) disable=1
	tStr = "*_tb"+num2str(tab)
	ModifyControlList ControlNameList("",";",tStr) disable=0
	ModifyControlList ControlNameList("",";","tb*") disable=0
			
	// disable select controls on given tab
	
	switch(tab)
		case 0:
			Button tb_doit, title=udFLpan00, userData="Load"
			ControlInfo flist_tb0
			if (cmpstr(S_value,"Create New")!=0)
				SetVariable newdir_tb0 disable=1
				TitleBox fldrx_tb0 disable=1
			else
				ControlInfo newdir_tb0
				tStr = "root:" + S_value
				TitleBox fldrx_tb0 disable=(!DataFolderExists(tStr))
				SetVariable newdir_tb0 disable=0
			endif
			ControlInfo fname_tb0
			SetVariable newfname_tb0 disable=((V_Value!=4) && (V_Value!=6))
			ControlInfo flist_tb1
			if (strlen(FunctionList("udFLPanelSetTab1",";",""))!=0)
				Execute/Q "udFLPanelSetTab1(0)"
			endif
			break
		default:
			sprintf tStr, "udFLPanelSetTab%d", tab
			if (strlen(FunctionList(tStr,";",""))!=0)
				sprintf tStr, "%s(1)", tStr
				Execute/Q tStr
			endif
			break
	endswitch
	
	SubPanelShow(tab)
	
	return 0

End

// show or kill the file list panel

Static Function SubPanelShow(tab)
	variable tab
	
//	STRUCT udFLPanelStructure sLP
	
	wave/T flist = root:Packages:StandardizedFileLoader:filesindatapath
	wave slist =root:Packages:StandardizedFileLoader:nfilesindatapath
	
	string lfname=""
		
	if (tab==0)
	
		// switch from something to load
		
		PathInfo udFL_DataFiles
		if (V_flag)
			lfname = GetUserData("","setpaths_tb0","")
			if (cmpstr(lfname,"1")==0)
				ControlInfo loader_tb0
				lfname = StringFromList(V_value-1,udFLLoaderList(1))
				GetFilesInDataDirPath(lfname)
				Button setpaths_tb0, userData="0"				
			endif
			if (strlen(ChildWindowList("#"))==0)
//				udFLStandardPanel#Initialize(sLP)
#if(exists("ScreenSizer#Initialize")==6)
				ScreenSizer#MakeExtPanel(2,0,SPWidth,SPHeight,"File List",other="/HOST=#/N=udFL_FileListPanel/K=2")
#else
				NewPanel/W=(0,0,SPWidth,SPHeight)/HOST=#/N=udFL_FileListPanel/K=2/EXT=2 as "File List"
#endif
				ListBox theList,pos={4,4},size={0.9*SPWidth,0.9*SPHeight}, mode=4, listwave=flist, selWave=slist
				SetActiveSubWindow ##
			endif
			Button tb_doit, disable=0
		else
			Button tb_doit, disable=2
			if (strlen(ChildWindowList("#"))!=0)
				KillWindow #udFL_FileListPanel
				Button setpaths_tb0, title=udFLpan00i1, userData="0"
			endif
		endif
	else
	
		// switch from load to something else
			
		if (strlen(ChildWindowList("#"))!=0)
			KillWindow #udFL_FileListPanel
		endif

	endif
	
	return 0
end

// load data

Static Function udFL_LoadData()
	
	STRUCT udFLStandardStructure udFL
		
	wave wnfiles=root:Packages:StandardizedFileLoader:nfilesindatapath
	wave/T files=root:Packages:StandardizedFileLoader:filesindatapath
	
	SVAR ltf = root:Packages:StandardizedFileLoader:loadtofolder
	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	SVAR uD = root:Packages:StandardizedFileLoader:userData
	SVAR lfp = root:Packages:StandardizedFileLoader:loadfileprefix
	SVAR llowl = root:Packages:StandardizedFileLoader:listofwavesloaded
	
	// no data path or no files
//	
//	if (numpnts(wnfiles)==0)
//		ControlInfo lmode_tb0
//		if (V_value==1)
//			DoAlert 0,udFLdb00a
//			return 0
//		endif
//	endif
	
	if (sum(wnfiles)==0)
		ControlInfo lmode_tb0
		if (V_value==1)
			DoAlert 0, udFLdb00b
			return 0
		endif
	endif
		
	string floadname = lFN
	variable ic, nc=numpnts(wnfiles)
	
	// initialize the loader
	
	udFLInitializeLoader(udFL,floadname)
	
	// set the Inputs
	
	ControlInfo lmode_tb0
	udFL.userLCtrl = V_value
	ControlInfo pmode_tb0
	udFL.userPCtrl = V_value
	udFL.userData = uD
	ControlInfo rpmode_tb0
	udFL.reportCtrl = V_value-2
	ControlInfo fname_tb0
	udFL.setNames = 2^(V_value-1)
	if ((V_value==4)||(V_value==6))
		udFL.namePrefix = lfp
	endif
	
	// set the Localizers

	PathInfo udFL_DataFiles
	udFL.pathStr = S_path
	udFL.returnCtrl = 0
	udFL.toFolder = ltf
	
	for (ic=0;ic<nc;ic+=1)
		if (wnfiles[ic])
			udFL.fileList += files[ic] + ";"
		endif
	endfor
	
	// load the files

	string cdf = GetDataFolder(1)
	
	if (udFL.reportCtrl==1)
		print "\r---\rUsing ... ", floadname
	endif
	
	if (udFLRunLoader(udFL,floadname)<0)
		SetDataFolder $cdf
		udFLHandleError(udFL)
		return -1
	else
		udFLHandleError(udFL)
	endif
	
	nc = udFL.itemsCount
	string theItem
	llowl = ""
	for (ic=0;ic<nc;ic+=1)
		theItem = "root:" + udFL.toFolder + ":" + StringFromList(ic,udFL.itLst.WAVES)
		if (exists(theItem)==1)
			llowl += StringFromList(ic,udFL.itLst.WAVES) + ";"
		endif
	endfor
	
	SetDataFolder $cdf
		
	return 0

End

// CONTROLS

// Tabs

// show a tab when it is clicked

Static Function ShowPanelTab(TC)
	STRUCT WMTabControlAction &TC
	
	switch(TC.eventcode)
		case 2:
			PanelFillTab(TC.tab)
			break
	endswitch
	
	return 0
End

// Buttons

// set paths

Static Function SetDirPaths(bc) : ButtonControl
	STRUCT WMButtonAction &bc

	wave slist =root:Packages:StandardizedFileLoader:nfilesindatapath
	SVAR thePath = root:Packages:StandardizedFileLoader:udFLdatapath
	
	switch(bc.eventCode)
		case 2:
			PathInfo udFL_DataFiles
			if (!V_flag)
				NewPath/Q udFL_DataFiles, SpecialDirPath("Desktop",0,0,0)
			endif
			PathInfo/S udFL_DataFiles
			ControlInfo rpmode_tb0
			if (V_value==1)
				NewPath/Q/O/M=udFLpan00k udFL_DataFiles
			else
				NewPath/O/M=udFLpan00k udFL_DataFiles			
			endif
			if (V_flag)
				return 0
			endif
			PathInfo/S udFL_DataFiles
//			V_flag = ItemsInList(S_path,":")
			thePath = StringFromList(ItemsInList(S_path,":")-1,S_path,":") 
			Button setpaths_tb0, title=udFLpan00i2, userdata="1"
			SubPanelShow(0)
//			if ((numpnts(slist)!=0) && (sum(slist)!=0))
//				Button tb_doit, disable=0
//			else
//				Button tb_doit, disable=2
//			endif
			break
	endswitch
	
	return 0
end

// load or display data

Static Function DoIt(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	SVAR llowl = root:Packages:StandardizedFileLoader:listofwavesloaded
	SVAR lfn = root:Packages:StandardizedFileLoader:loadfunctionname
	
	variable cm
	
	switch( ba.eventCode )
		case 2:
		
			ControlInfo tb_closeme
			cm = V_Value
					
			strswitch(ba.UserData)
			
				// load
				
				case "Load":
//					ControlInfo fname_tb0
//					lnu = V_value
					ControlInfo loader_tb0
					lfn = StringFromList(V_value-1,udFLLoaderList(1),";")
					llowl = ""
					udFL_LoadData()
					ControlInfo flist_tb0
					TitleBox fldrx_tb0 disable=(cmpstr(S_value,"Create New"))
					break
				default:
					execute/Q ba.UserData
					break
			endswitch
			if (cm==1)
				DoWindow/K udFL_Panel
			endif
	endswitch

	return 0
End

// Set Variables

// validate new folder string

Static Function ValidateLoadFolderName(sv) : SetVariableControl
	STRUCT WMSetVariableAction &sv

	SVAR lff = root:Packages:StandardizedFileLoader:loadtofolder
	
	variable trn
	string theF
	
	switch(sv.eventCode)
		case 2:
			if (strsearch(sv.sval," ",0)>0)
				lff = "INVALID Spaces!"
				ControlUpdate newdir_tb0
				trn = ticks
				do
				while((ticks-trn)<90)
				lff=""
				SetVariable newdir_tb0 activate
			endif
			theF = "root:" + lff
			TitleBox fldrx_tb0 disable=(!DataFolderExists(theF))
			break	
	endswitch
	
	return 0
	
End

// validate new file string

Static Function ValidateNewTraceName(sv) : SetVariableControl
	STRUCT WMSetVariableAction &sv

	SVAR lfp = root:Packages:StandardizedFileLoader:loadfileprefix
	
	switch(sv.eventCode)
		case 2:
			lfp = CleanupName(lfp,0)
			SetVariable newfname_tb0 activate
			break	
	endswitch
	
	return 0
End

// Checkboxes

// PopupMenu

// change loader modes (and zero the userData) when loader is changed

Static Function ChangeLoaderModes(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR uD = root:Packages:StandardizedFileLoader:userData
	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	string lFNa
	
	switch(pa.eventCode)
		case 2:
			lFNa = "udStFiLr" + pa.popStr		
			if (cmpstr(lFNa,lfN)==0)
				return 0
			endif
			uD = ""
			lFN = lfNa		
			PopupMenu lmode_tb0,mode=1,value=#"udFLStandardPanel#FileLoaderModeList()"
			PopupMenu pmode_tb0,mode=1,value=#"udFLStandardPanel#FileProcessModeList()"
			PopupMenu fname_tb0,mode=1,value=#"udFLStandardPanel#RenameInputList()"
			SetVariable newfname_tb0, disable=1
			if (strlen(ChildWindowList("#"))!=0)
				GetFilesInDataDirPath(lFNa)
			endif
	endswitch
	
	return 0
End

// potentially create a new directory name

Static Function CreateNewFolderName(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR ltf = root:Packages:StandardizedFileLoader:loadtofolder
	
	switch(pa.eventCode)
		case 2:
			TitleBox fldrx_tb0 disable=1
			if (pa.popNum < 3)
				PopupMenu flist_tb0 mode=1
				SetVariable newdir_tb0 disable=0
				ltf = ""
			else
				SetVariable newdir_tb0 disable=1
				ltf = pa.popStr
			endif
	endswitch
	
	return 0
End

// potentially create a new trace name

Static Function CreateNewTraceName(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	
	switch(pa.eventCode)
		case 2:
			SetVariable newfname_tb0 disable=((pa.popNum!=4)&&(pa.popNum!=6))
	endswitch
	
	return 0
End

Static Function/S FolderList()

	return (udFLpan00b + ";-;" + udFLFolderList())

end

// List Functions

Static Function/S RenameInputList()

	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	return (udFLQuerysetNames(lFN))
end


Static Function/S FileLoaderModeList()

	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	return (udFLQueryLoadModes(lFN))

End

Static Function/S FileProcessModeList()

	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	return (udFLQueryProcModes(lFN))

End

Static Function GetFilesInDataDirPath(lfname)
	string lfname

	wave/T flist = root:Packages:StandardizedFileLoader:filesindatapath
	wave nlist =root:Packages:StandardizedFileLoader:nfilesindatapath

	STRUCT udFLStandardStructure udFL
	udFLInitializeLoader(udFL,lfname)
	
	string mimetype = udFL.mimetype
	string filetypes = udFL.extensions
	
	string lofiles = "", nlof = ""
	variable nfiles, ic, ntypes
	
	lofiles = GrepList(IndexedFile(udFL_DataFiles,-1,"????"),"^\.+",1,";")
	nfiles = ItemsInList(lofiles,";")
	ntypes = ItemsInList(filetypes,";")

	for (ic=0;ic<ntypes;ic+=1)
		nlof += ListMatch(lofiles,("*" + StringFromList(ic,filetypes)))
	endfor
	
	nfiles = ItemsInList(nlof,";")
	redimension/N=(nfiles) flist
	redimension/N=(nfiles) nlist
	flist = StringFromList(p,nlof,";")
	nlist = 0
	
	return 0
end
